// part of KSUDOKU - by Francesco Rossi <redsh@email.it> 2005

#include "ksudoku.h"

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <ksavefile.h>

#include <kmessagebox.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kstatusbar.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kactionclasses.h>
#include <krun.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kurlrequesterdlg.h>

#include <knewdlg.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

void ksudoku::updateStatusBar()
{
	int secs = m_view->t.elapsed() / 1000;
	int mins = secs / 60;
	secs = secs % 60;
	
	QString m;

	if(secs % 36 < 12)
		m.sprintf("Selected item %c, Time elapsed %d:%02d. Press SHIFT to highlight.", m_view->puzzle->zerochar +m_view->current_selected_number, mins, secs);
	else  if(secs % 36 >=12 &&  secs % 36 < 24)
		m.sprintf("Selected item %c, Time elapsed %d:%02d. Use RMB to pencil-mark (superscript).", m_view->puzzle->zerochar +m_view->current_selected_number, mins, secs);
	else
		m.sprintf("Selected item %c, Time elapsed %d:%02d. Type in a cell to place that number in it.", m_view->puzzle->zerochar +m_view->current_selected_number, mins, secs);


	statusBar()->message(m);
}

void ksudoku::load(const KURL& url)
{
	if (!myURL.isEmpty() && myURL.isValid())
	{
		QString name;
		KIO::NetAccess::download(url, name, this);
			
		FILE *fp;
		
		if (!(fp = fopen(QFile::encodeName(name), "r"))) 
		{
			KMessageBox::information(this, "Could not open file.");
			return;
		}

		fscanf(fp, "%d\n", &order);
				

		if((order != 9 && order != 16)) return;
		m_view->setup(order,0,true);
			
		ITERATE(i,order)
		{
		
			unsigned char b;
			ITERATE(j,order)
			{
				fscanf(fp, "%c", &b);
				m_view->puzzle->numbers[i*order+j] = (unsigned) b-m_view->puzzle->zerochar;
				if(m_view->puzzle->numbers[i*order+j] !=0) m_view->btns[i*order+j]->given = true;
				m_view->btns[i*order+j]->setValue ( m_view->puzzle->numbers[i*order+j]);
				m_view->btns[i*order+j]->update();
			}
			fscanf(fp, "%c", &b);
		}

		//ITERATE(i,order*order) KMessageBox::information(this, QString::number((m_view->btns[i]->value)));
		m_view->update();
		//m_view->resizeEvent(0);
		stateChanged("dubbing");
		setCaption(url.prettyURL());
		
		KIO::NetAccess::removeTempFile( name );
	}
}

ksudoku::ksudoku(bool dub2, bool load, KURL path)
    : KMainWindow( 0, "ksudoku" ),
      m_view(new ksudokuView(this)),
      m_printer(0)
{
    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
	readProperties( KApplication::kApplication()->config());
    setupActions();

    // and a status bar
    statusBar()->show();

    // Apply the create the main window and ask the mainwindow to
		// automatically save settings if changed: window size, toolbar
    // position, icon size, etc.  Also to add actions for the statusbar
		// toolbar, and keybindings if necessary.
    setupGUI();

    // allow the view to change the statusbar and caption

	if(!load)
	{	
		dub = dub2;
	
		if(dub)
			stateChanged("dubbing");
		else
			stateChanged("dubbing", StateReverse);
			
		KNewDlg w(dub);
		w.setGeometry(100,100,200,100);
		w.exec();
				
		if(w.success == true)
		{
			order =  w.order;
			difficulty = w.difficulty;
			m_view->setup(order, difficulty, dub);
		}
		else
		{
			return;
		}				
	}
	else
	{
		dub = true;
		myURL = path;
		this->load(path);
	}

	QTimer *timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), this, SLOT(updateStatusBar()) );
    timer->start( 1000, false ); // 2 seconds single-shot timer
    connect( m_view, SIGNAL(changedSelectedNum()), this, SLOT(updateStatusBar()) );
}

ksudoku::~ksudoku()
{

}
	void ksudoku::mouseOnlySuperscript()
	{
		if(m_view->mouseOnlySuperscript == 0)
			m_view->mouseOnlySuperscript = 1;
		else
			m_view->mouseOnlySuperscript = 0;

		saveProperties( KApplication::kApplication()->config());
	}
	void ksudoku::guidedMode()
	{
		ITERATE(i,m_view->puzzle == 0) return;
		if(m_view->guidedMode ==  0)
			m_view->guidedMode = 1;
		else
			m_view->guidedMode = 0;

		ITERATE(i,m_view->puzzle->size)
			m_view->btns[i]->paintEvent(0);

		saveProperties( KApplication::kApplication()->config());
	}

	void ksudoku::homepage()
	{
		KRun::runURL (KURL("http://ksudoku.sourceforge.net/"), "text/html");
	}
	void ksudoku::support()
	{
		KRun::runURL (KURL("http://sourceforge.net/project/project_donations.php?group_id=147876"), "text/html");
	}
	void ksudoku::sendComment()
	{
		KRun::runURL (KURL("http://ksudoku.sourceforge.net/newcomment.php"), "text/html");
	}

	void ksudoku::checkForUpdates()
	{
		QString name;
		char buf[16];
		QString myVer = "0.2";
		KIO::NetAccess::download(KURL("http://ksudoku.sourceforge.net/latest.php"), name, this);
			
		FILE *fp;
		
		if (!(fp = fopen(QFile::encodeName(name), "r"))) 
		{
			KMessageBox::information(this, "Could not get the response from server.");
			return;
		}

		fscanf(fp, "%s", buf);
		if(QString(&buf[0]) == myVer)
			KMessageBox::information(this, "Your program is at the latest version");
		else
		{
			QString msg;
			msg.sprintf("Your program version is %s, the latest version is %s.\nDo you want to update?",  myVer.ascii(), &buf[0]);
			if(KMessageBox::questionYesNo(this, msg) == KMessageBox::Yes)
				KRun::runURL (KURL("http://ksudoku.sourceforge.net/3.htm"), "text/html");
		}
		//close(fp);
		KIO::NetAccess::removeTempFile( name );
	}
	void ksudoku::giveHint()
	{
		printf("Remaining %d\n",m_view->remaining);
		if(m_view->remaining == 0) return;
		m_view->remaining--;

		int i;
		do
			i = random() % m_view->puzzle->size;		
		while(m_view->puzzle->numbers[i] != 0);
		
		m_view->puzzle->numbers[i] = m_view->completed_puzzle->numbers[i] ;
	    m_view->btns[i]->given = true;
		m_view->btns[i]->setValue(m_view->puzzle->numbers[i]);
		
		m_view->btns[i]->update();
		update();
	}
	
	void ksudoku::autoSolve()
	{
		//m_view->mySolver->solve(m_view->puzzle, 1, m_view->puzzle);
		for(int i=0; i<m_view->puzzle->order; i++)
			for(int j=0; j<m_view->puzzle->order; j++)
			{
				if(m_view->completed_puzzle->numbers[i] !=0) m_view->btns[i*m_view->puzzle->order+j]->given = true;
				m_view->btns[i*m_view->puzzle->order+j]->setValue(m_view->completed_puzzle->numbers[i*m_view->puzzle->order+j]);
				m_view->btns[i*m_view->puzzle->order+j]->update();
			}
		m_view->remaining= 0;
		update();
	}
	
	void ksudoku::dubPuzzle()
	{
		if(!dub)
		{
			(new ksudoku(true))->show();
		}
		else
		{
			ITERATE(i,m_view->puzzle->size)
				m_view->puzzle->numbers[i] = m_view->btns[i]->value;
				
			m_view->genPuzzle(0, true);
			m_view->update();
			stateChanged("dubbing", StateReverse);
		}
	}
	
	void ksudoku::genMultiple()
	{
		KMessageBox::information(this, i18n("Sorry, this feature is under development."));
	}
	
	void ksudoku::set0(){m_view->current_selected_number = 0;updateStatusBar();}
	void ksudoku::set1(){m_view->current_selected_number = 1;updateStatusBar();}
	void ksudoku::set2(){m_view->current_selected_number = 2;updateStatusBar();}
	void ksudoku::set3(){m_view->current_selected_number = 3;updateStatusBar();}
	void ksudoku::set4(){m_view->current_selected_number = 4;updateStatusBar();}
	void ksudoku::set5(){m_view->current_selected_number = 5;updateStatusBar();}
	void ksudoku::set6(){m_view->current_selected_number = 6;updateStatusBar();}
	void ksudoku::set7(){m_view->current_selected_number = 7;updateStatusBar();}
	void ksudoku::set8(){m_view->current_selected_number = 8;updateStatusBar();}
	void ksudoku::set9(){m_view->current_selected_number = 9;updateStatusBar();}
	void ksudoku::set10(){m_view->current_selected_number = 10;updateStatusBar();}
	void ksudoku::set11(){m_view->current_selected_number = 11;updateStatusBar();}
	void ksudoku::set12(){m_view->current_selected_number = 12;updateStatusBar();}
	void ksudoku::set13(){m_view->current_selected_number = 13;updateStatusBar();}
	void ksudoku::set14(){m_view->current_selected_number = 14;updateStatusBar();}
	void ksudoku::set15(){m_view->current_selected_number = 15;updateStatusBar();}
	void ksudoku::set16(){m_view->current_selected_number = 16;updateStatusBar();}
	
void ksudoku::setupActions()
{
    KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
    KStdAction::open(this, SLOT(fileOpen()), actionCollection());
    KStdAction::save(this, SLOT(fileSave()), actionCollection());
    KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
    KStdAction::print(this, SLOT(filePrint()), actionCollection());
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());

    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
				  						 
								  
	(void)new KAction(i18n("del"), 0, this, SLOT(set0()), actionCollection(), "0");
	(void)new KAction(i18n("1"), 0, this, SLOT(set1()), actionCollection(), "1");
	(void)new KAction(i18n("2"), 0, this, SLOT(set2()), actionCollection(), "2");
	(void)new KAction(i18n("3"), 0, this, SLOT(set3()), actionCollection(), "3");
	(void)new KAction(i18n("4"), 0, this, SLOT(set4()), actionCollection(), "4");
	(void)new KAction(i18n("5"), 0, this, SLOT(set5()), actionCollection(), "5");
	(void)new KAction(i18n("6"), 0, this, SLOT(set6()), actionCollection(), "6");
	(void)new KAction(i18n("7"), 0, this, SLOT(set7()), actionCollection(), "7");
	(void)new KAction(i18n("8"), 0, this, SLOT(set8()), actionCollection(), "8");
	(void)new KAction(i18n("9"), 0, this, SLOT(set9()), actionCollection(), "9");
	(void)new KAction(i18n("l"), 0, this, SLOT(set10()), actionCollection(), "l");
	(void)new KAction(i18n("m"), 0, this, SLOT(set11()), actionCollection(), "m");
	(void)new KAction(i18n("n"), 0, this, SLOT(set12()), actionCollection(), "n");
	(void)new KAction(i18n("o"), 0, this, SLOT(set13()), actionCollection(), "o");
	(void)new KAction(i18n("p"), 0, this, SLOT(set14()), actionCollection(), "p");
	(void)new KAction(i18n("q"), 0, this, SLOT(set15()), actionCollection(), "q");
	(void)new KAction(i18n("r"), 0, this, SLOT(set16()), actionCollection(), "r");
	
	(void)new KAction(i18n("a"), 0, this, SLOT(set1()), actionCollection(), "a");
	(void)new KAction(i18n("b"), 0, this, SLOT(set2()), actionCollection(), "b");
	(void)new KAction(i18n("c"), 0, this, SLOT(set3()), actionCollection(), "c");
	(void)new KAction(i18n("d"), 0, this, SLOT(set4()), actionCollection(), "d");
	(void)new KAction(i18n("e"), 0, this, SLOT(set5()), actionCollection(), "e");
	(void)new KAction(i18n("f"), 0, this, SLOT(set6()), actionCollection(), "f");
	(void)new KAction(i18n("g"), 0, this, SLOT(set7()), actionCollection(), "g");
	(void)new KAction(i18n("h"), 0, this, SLOT(set8()), actionCollection(), "h");
	(void)new KAction(i18n("i"), 0, this, SLOT(set9()), actionCollection(), "i");

	(void)new KAction(i18n("Give Hint!"),  QIconSet( QPixmap("open.xpm")),  0, this, SLOT(giveHint()), actionCollection(), "giveHint"); //DONE
	(void)new KAction(i18n("Solve!"), 0, this, SLOT(autoSolve()), actionCollection(), "autoSolve");	//DONE
	(void)new KAction(i18n("Rip Puzzle"), 0, this, SLOT(dubPuzzle()), actionCollection(), "dubPuzzle");	
	//(void)new KAction(i18n("Generate Multiple"), 0, this, SLOT(genMultiple()), actionCollection(), "genMultiple");
	(void)new KAction(i18n("Check"), 0, this, SLOT(dubPuzzle()), actionCollection(), "FinishDubPuzzle");

	//WEB
	(void)new KAction(i18n("Check for updates"), 0, this, SLOT(checkForUpdates()), actionCollection(), "checkForUpdates");
	(void)new KAction(i18n("Home page"), 0, this, SLOT(homepage()), actionCollection(), "Home_page");
	(void)new KAction(i18n("Support this project"), 0, this, SLOT(support()), actionCollection(), "support");
	(void)new KAction(i18n("Send comment"), 0, this, SLOT(sendComment()), actionCollection(), "SendComment");

	//Settings
	KToggleAction* a = new KToggleAction(i18n("Mouse-Only superscript mode"), 0, this, SLOT(mouseOnlySuperscript()), actionCollection(), "mouseOnlySuperscript");
	a->setChecked(m_view->mouseOnlySuperscript);
	
	a=new KToggleAction(i18n("Guided Mode (mark wrong red)"), 0, this, SLOT(guidedMode()), actionCollection(), "guidedMode");
	a->setChecked(m_view->guidedMode);

	//Checkpoints
	(void)new KAction(i18n("Push checkpoint (add)"), 0, m_view, SLOT(push()), actionCollection(), "pushC");
	(void)new KAction(i18n("Pop checkpoint (extract)"), 0, m_view, SLOT(pop()), actionCollection(), "popC");
}


void ksudoku::saveProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored

/*#if KDE_IS_VERSION(3,1,3)
        config->writeEntry("lastURL", m_view->currentURL());
        config->writeEntry("guidedMode", m_view->guidedMode);
        config->writeEntry("mouseOnlySuperscript", m_view->mouseOnlySuperscript);
#else*/
        config->writeEntry("lastURL", m_view->currentURL());
        config->writeEntry("guidedMode", m_view->guidedMode);
 		config->writeEntry("mouseOnlySuperscript", m_view->mouseOnlySuperscript);
	 	config->writeEntry("FIRSTRUN", 1);
		config->sync();
//#endif
}

void ksudoku::readProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'

    QString url = config->readEntry("lastURL");
	if(config->readBoolEntry("FIRSTRUN") ==0)
	{	
		m_view->guidedMode=1;
 		m_view->mouseOnlySuperscript=1;
		saveProperties(config);
		return;
	}
    m_view->guidedMode = config->readBoolEntry("guidedMode");
    m_view->mouseOnlySuperscript = config->readBoolEntry("mouseOnlySuperscript");

	/*if (!url.isEmpty())
        m_view->openURL(KURL(url));*/
}

void ksudoku::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    event->accept(KURLDrag::canDecode(event));
}

void ksudoku::dropEvent(QDropEvent *event)
{
    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
    KURL::List urls;

    // see if we can decode a URI.. if not, just ignore it
    if (KURLDrag::decode(event, urls) && !urls.isEmpty())
    {
        // okay, we have a URI.. process it
        const KURL &url = urls.first();

        // load in the file
        load(url);
    }
}

void ksudoku::fileNew()
{
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

  
    // create a new window
    (new ksudoku)->show();
}

void ksudoku::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
    // standard filedialog
    myURL = KFileDialog::getOpenURL(QString::null, QString::null, this, i18n("Open Location"));
	(new ksudoku(false, true, myURL))->show();		
	
}

void ksudoku::fileSave()
{
    // this slot is called whenever the File->Save menu is selected,
    // the Save shortcut is pressed (usually CTRL+S) or the Save toolbar
    // button is clicked

    // save the current file

	if(m_view == 0) return;
	if(m_view->puzzle==0) return;
	
	if(myURL.isEmpty())	myURL = KFileDialog::getSaveURL();
    if (!myURL.isEmpty() && myURL.isValid())
	{
		KSaveFile f(myURL.path());
		if(f.status() == 0)
		{
			QTextStream *s = f.textStream();
			*s << m_view->puzzle->order;
			*s << "\n";
			ITERATE(i, m_view->puzzle->size)
			{
				*s << (char) (m_view->puzzle->zerochar + m_view->puzzle->numbers[i]);
				if(i % m_view->puzzle->order == m_view->puzzle->order-1) *s << '\n';
			}
			f.close();
		}
		else
			KMessageBox::information(this, "Could not write to file.");
	}
}

void ksudoku::fileSaveAs()
{
    // this slot is called whenever the File->Save As menu is selected,
    myURL = KFileDialog::getSaveURL();
    if (!myURL.isEmpty() && myURL.isValid())
    	fileSave();
}

void ksudoku::filePrint()
{
    // this slot is called whenever the File->Print menu is selected,
    // the Print shortcut is pressed (usually CTRL+P) or the Print toolbar
    // button is clicked
    if (!m_printer) m_printer = new KPrinter;
    if (m_printer->setup(this))
    {
        // setup the printer.  with Qt, you always "print" to a
        // QPainter.. whether the output medium is a pixmap, a screen,
        // or paper
        QPainter p;
        p.begin(m_printer);

        // we let our view do the actual printing
        QPaintDeviceMetrics metrics(m_printer);
        m_view->print(&p, metrics.height(), metrics.width());

        // and send the result to the printer
        p.end();
    }
}

void ksudoku::optionsPreferences()
{
    // popup some sort of preference dialog, here
/*    ksudokuPreferences dlg;
    if (dlg.exec())
    {
        // redo your settings
    }*/
}

void ksudoku::changeStatusbar(const QString& text)
{
    // display the text on the statusbar
    statusBar()->message(text);
}


void ksudoku::changeCaption(const QString& text)
{
    // display the text on the caption
    setCaption(text);
}
#include "ksudoku.moc"

