// part of KSUDOKU - by Francesco Rossi <redsh@email.it> 2005
#include "sudoku_solver.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
//#include "sudoku_types.h"

// SUDOKU PUZZLE SOLVER BASIC ALGORITHM - BY FRANCESCO ROSSI 2005 redsh@email.it

skSolver::skSolver(int n)
{
	order=n;
	size = order*order;
	
}

skSolver::~skSolver()
{
	delete g;
}

int skSolver::remove_numbers(skPuzzle* p, int level)
{
	int  b;
	int cnt=p->size;
	int to=p->size*3;
	int solutions_d=0;
	skPuzzle stack[257];
	bool done[257];
	ITERATE(i,p->size+1){ stack[i].setorder(order); done[i]=false;};
		
	if(p->order>9) to = p->size-32;
	skPuzzle c(p->order);
	copy(&c, p);
	
	if(level < 0 && p->order==9)
	{
		int val = RANDOM(p->order)+1;
		ITERATE(i,size)
			if(p->numbers[i] == val) p->numbers[i] = 0;
	}

	ITERATE(q, to) 
	{
		int idx = RANDOM(p->size);  //2FIX
		
		int index = idx;
		b = p->numbers[index];
		printf("%d/%d\n", q,to);
		if(b!=0) if (done[index] == false)
		{
			done[index] = true;
			solutions_d=0;
			p->numbers[index] = 0;
			copy(&stack[0], p);

			solve_engine(&stack[0], solutions_d, 0, 2, index, index, b);
			if(solutions_d == 1)
				cnt--;
			else
				p->numbers[index] = b;
		} 
	}
	
	
	ITERATE(i, level*(p->order+LEVINC))
	{
		int idx = RANDOM(p->size);//2FIX
		while(p->numbers[idx] != 0) idx=(idx+1) % p->size;
		p->numbers[idx] = c.numbers[idx];
		cnt++;
	}
	
	/* GENERATES PUZZLES WITH MULTIPLE SOLUTIONS
	if(level < 0)
	{
		ITERATE(i,2)
		{
			int idx = RANDOM(p->size);//2FIX
			while(p->numbers[idx] == 0) idx=(idx+1) % p->size;
			p->numbers[idx] = 0;
			cnt--;
		}
	}*/
	
	return cnt;
}

skPuzzle* head;
int skSolver:: solve(skPuzzle* puzzle, int max_solutions, skPuzzle* out_solutions)
{
	if(puzzle->order != order) return -1;
	if(g == 0) return -2;
	
	int solutions_d=0;
	skPuzzle stack[257];
	head = &stack[0];
	ITERATE(i,puzzle->size+1) stack[i].setorder(order);
	copy(&stack[0], puzzle);
	
	solve_engine(&stack[0], solutions_d, out_solutions, max_solutions, -1);
	
	return solutions_d;
}

int skSolver:: solve_engine(skPuzzle *s,  int& solutions, skPuzzle* solution_list, int maxsolutions, int last_add,	 int dynindex, int dynvalue) //last_add for further optimizations
{
	if(maxsolutions>0 && solutions>=maxsolutions)	return 0;
	
	if(dynindex!=-1)if(dynvalue == s->numbers[dynindex])
	{
			solutions++;
			return 1;
	}

	int lowest_pos,lowest,lowest_val;
	lowest = s->order+1;
		
	for(int i=last_add*(last_add != -1); i<(last_add+1)+s->size*(last_add == -1); i++)
		if(s->numbers[i] != 0)
			ITERATE(j,g->optimized_d[i])
				if(s->numbers[g->optimized[i][j]] == 0)
					s->flags[g->optimized[i][j]][s->numbers[i]] = 0;
		
	for(int q=0; (last_add==-1) ? q<s->size : q<g->optimized_d[last_add]; q++)
	{
		int i = (last_add==-1) ? q : g->optimized[last_add][q];
		if(s->numbers[i] == 0)
		{
			int c=0;
			ITERATE(j,s->order)
				c+=s->flags[i][j+1];
	
			if(c<lowest)
			{
				lowest_pos = i;
				lowest     = c;

				if(lowest < 1)
					return -1;
				if(lowest == 1)
				{
					ITERATE(j,s->order) if(s->flags[lowest_pos][j+1] == 1) lowest_val = j+1;
					//ITERATE(j, s-head) printf(" "); printf("ADDED %d %d\n", lowest_pos, lowest_val);
					s->numbers[lowest_pos] = lowest_val;
					return solve_engine(s, solutions, solution_list, maxsolutions, lowest_pos);
				}
			}
		}
	}
	
	if(last_add != -1) return solve_engine(s, solutions, solution_list, maxsolutions, -1);
			
	//check completed
	int remaining=0;
	ITERATE(i,s->size)if(s->numbers[i] == 0)remaining++;
	
	if(remaining == 0)
	{
		if(solution_list) 
			if(&solution_list[solutions]) 
				copy(&solution_list[solutions], s);
		solutions++;
	
		/*{
			printf("%d Solution found TC\n", solutions);
			ITERATE(i,s->size)
			{
				printf("%c", zerochar  + s->numbers[i]);
				if(i%s->order == s->order-1) printf("\n");
			}
		}*/
		return 1;
	}
	
	if(remaining == 1) return -1;
	
	//fork on lowest if not added
	int positions[17]; //2fix
	int positions_d = 0;
		
	ITERATE(i, s->order)
		if(s->flags[lowest_pos][i+1])
			positions[positions_d++] = i+1;
			
	while(positions_d>0)
	{	
		copy(&s[1], s);
		int index = RANDOM(positions_d);
		s[1].numbers[lowest_pos] = positions[index];
		solve_engine(&s[1], solutions, solution_list, maxsolutions, lowest_pos);
			
		for(int i=index; i<positions_d-1; i++) positions[i] = positions[i+1];
		positions_d--;
	}
	
	return -1;	
}


int skSolver:: init()
{
	g = new skGraph(order);
	
	int row,col, subsquare;
	ITERATE(i,g->size)
	{
		ITERATE(j,g->order) g->graph[i][j] = 0;
		row       = i / g->order;
		col       = i % g->order;
		subsquare = col/g->base+(row/g->base)*g->base;
		
		ITERATE(j,g->order)
		{
			g->graph[i][j+row*g->order]  = 1;
			g->graph[i][j*g->order+col]  = 1;
			g->graph[i][((subsquare/g->base)*g->base + j%g->base) * g->order + 
			(subsquare%g->base)*g->base + j/g->base] = 1;
		}
	}

	
	ITERATE(i,g->size)
	{
		g->optimized_d [i] = 0;
		ITERATE(j,size)
			if(g->graph[i][j]==1)
			{
				g->optimized[i][g->optimized_d [i]]=(char) j;
				g->optimized_d [i]++;
			}
	}

	if(order>9) zerochar = 'a'-1;
	else zerochar = '0';
	
	return 0;
}

void skSolver::copy(skPuzzle* dest, skPuzzle* src)
{
	dest->order = src->order;
	dest->base  = src->base;
	dest->size  = src->size;

	ITERATE(i, src->size)
	{
		dest->numbers[i] = src->numbers[i];
		ITERATE(j, src->order+1)
			dest->flags[i][j] = 1;//src->flags[i][j];
	}
	
}
